@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2007, 2009, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -98,6 +98,14 @@
 #define C(x) x, sizeof(x) -1
 #define S(x) x->str, x->len
 
+#ifndef PLUGIN_VERSION
+#ifdef CHASSIS_BUILD_TAG
+#define PLUGIN_VERSION PACKAGE_VERSION "." CHASSIS_BUILD_TAG
+#else
+#define PLUGIN_VERSION PACKAGE_VERSION
+#endif
+#endif
+
 struct chassis_plugin_config {
 	gchar *address;                   /**< listening address of the admin interface */
 
@@ -239,12 +247,12 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_read_auth) {
 	/* decode the packet */
 	network_mysqld_proto_skip_network_header(&packet);
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(con->client->challenge->capabilities);
 	if (network_mysqld_proto_get_auth_response(&packet, auth)) {
 		network_mysqld_auth_response_free(auth);
 		return NETWORK_SOCKET_ERROR;
 	}
-	if (!(auth->capabilities & CLIENT_PROTOCOL_41)) {
+	if (!(auth->client_capabilities & CLIENT_PROTOCOL_41)) {
 		/* should use packet-id 0 */
 		network_mysqld_queue_append(con->client, con->client->send_queue, C("\xff\xd7\x07" "4.0 protocol is not supported"));
 		network_mysqld_auth_response_free(auth);
@@ -263,12 +271,12 @@ NETWORK_MYSQLD_PLUGIN_PROTO(server_read_auth) {
 		con->state = CON_STATE_SEND_ERROR; /* close the connection after we have sent this packet */
 	} else if (network_mysqld_proto_password_hash(hashed_password, con->config->admin_password, strlen(con->config->admin_password))) {
 	} else if (network_mysqld_proto_password_scramble(excepted_response,
-				S(recv_sock->challenge->challenge),
+				S(recv_sock->challenge->auth_plugin_data),
 				S(hashed_password))) {
 		network_mysqld_con_send_error_full(send_sock, C("scrambling failed"), 1045, "28000");
 		
 		con->state = CON_STATE_SEND_ERROR; /* close the connection after we have sent this packet */
-	} else if (!g_string_equal(excepted_response, auth->response)) {
+	} else if (!g_string_equal(excepted_response, auth->auth_plugin_data)) {
 		network_mysqld_con_send_error_full(send_sock, C("password doesn't match"), 1045, "28000");
 		
 		con->state = CON_STATE_SEND_ERROR; /* close the connection after we have sent this packet */
@@ -574,6 +582,11 @@ static int network_mysqld_admin_plugin_apply_config(chassis *chas, chassis_plugi
 				G_STRLOC);
 		return -1;
 	}
+	if (!g_strcmp0(config->admin_password, "")) {
+		g_critical("%s: --admin-password cannot be empty",
+				G_STRLOC);
+		return -1;
+	}
 	if (!config->lua_script) {
 		g_critical("%s: --admin-lua-script needs to be set, <install-dir>/lib/mysql-proxy/lua/admin.lua may be a good value",
 				G_STRLOC);
@@ -619,7 +632,7 @@ static int network_mysqld_admin_plugin_apply_config(chassis *chas, chassis_plugi
 G_MODULE_EXPORT int plugin_init(chassis_plugin *p) {
 	p->magic        = CHASSIS_PLUGIN_MAGIC;
 	p->name         = g_strdup("admin");
-	p->version		= g_strdup(PACKAGE_VERSION);
+	p->version		= g_strdup(PLUGIN_VERSION);
 
 	p->init         = network_mysqld_admin_plugin_new;
 	p->get_options  = network_mysqld_admin_plugin_get_options;
