@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2007, 2009, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -52,6 +52,14 @@
 #define C(x) x, sizeof(x) - 1
 #define S(x) x->str, x->len
 
+#ifndef PLUGIN_VERSION
+#ifdef CHASSIS_BUILD_TAG
+#define PLUGIN_VERSION PACKAGE_VERSION "." CHASSIS_BUILD_TAG
+#else
+#define PLUGIN_VERSION PACKAGE_VERSION
+#endif
+#endif
+
 /**
  * we have two phases
  * - getting the binglog-pos with SHOW MASTER STATUS
@@ -223,9 +231,9 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_handshake) {
 	/* build the auth packet */
 	auth_packet = g_string_new(NULL);
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(shake->capabilities);
 
-	auth->capabilities = shake->capabilities;
+	auth->client_capabilities = shake->capabilities;
 	auth->charset      = shake->charset;
 
 	if (config->mysqld_username) {
@@ -238,7 +246,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(repclient_read_handshake) {
 		hashed_password = g_string_new(NULL);
 		network_mysqld_proto_password_hash(hashed_password, config->mysqld_password, strlen(config->mysqld_password));
 
-		network_mysqld_proto_password_scramble(auth->response, S(shake->challenge), S(hashed_password));
+		network_mysqld_proto_password_scramble(auth->auth_plugin_data, S(shake->auth_plugin_data), S(hashed_password));
 	}
 
 	network_mysqld_proto_append_auth_response(auth_packet, auth);
@@ -823,7 +831,7 @@ int network_mysqld_replicant_plugin_apply_config(chassis G_GNUC_UNUSED *chas, ch
 G_MODULE_EXPORT int plugin_init(chassis_plugin *p) {
 	p->magic        = CHASSIS_PLUGIN_MAGIC;
 	p->name         = g_strdup("replicant");
-	p->version		= g_strdup(PACKAGE_VERSION);
+	p->version		= g_strdup(PLUGIN_VERSION);
 	/* append the our init function to the init-hook-list */
 
 	p->init         = network_mysqld_replicant_plugin_init;
