@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2008, 2009, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -1259,7 +1259,9 @@ int main(int argc, char **argv) {
 	}
 
 	if (keyfile) {
-		if (chassis_keyfile_to_options(keyfile, "mysql-myisam-dump", main_entries)) {
+		if (FALSE == chassis_keyfile_to_options_with_error(keyfile, "mysql-myisam-dump", main_entries, &gerr)) {
+			g_critical("%s", gerr->message);
+
 			exit_code = EXIT_FAILURE;
 			goto exit_nicely;
 		}
