@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -31,6 +31,10 @@
 
 #include "glib-ext.h"
 
+#ifndef CLIENT_PLUGIN_AUTH
+#define CLIENT_PLUGIN_AUTH (1 << 19)
+#endif
+
 #define C(x) x, sizeof(x) - 1
 #define S(x) x->str, x->len
 
@@ -248,8 +252,8 @@ int network_mysqld_proto_get_com_query_result(network_packet *packet, network_my
 				   0x00 is part of the protocol for binary row packets
 				 */
 			} else {
-				/* the first field might be a NULL for a text row packet */
-				break;
+				/* fallthrough to default:
+				 * the first field might be a NULL for a text row packet */
 			}
 		default:
 			query->rows++;
@@ -548,23 +552,32 @@ int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld
 	case COM_CHANGE_USER: 
 		/**
 		 * - OK
-		 * - ERR (in 5.1.12+ + a duplicate ERR)
+		 * - ERR
+		 * - EOF for auth switch TODO
 		 */
 		err = err || network_mysqld_proto_get_int8(packet, &status);
 		if (err) return -1;
 
 		switch (status) {
 		case MYSQLD_PACKET_ERR:
-			is_finished = 1;
-			break;
 		case MYSQLD_PACKET_OK:
 			is_finished = 1;
 			break;
+		case MYSQLD_PACKET_EOF:
+			/* TODO:
+			 * - added extra states to the state-engine in network-mysqld.c to track the packets that are sent back and forth
+			 *   to switch the auth-method in COM_CHANGE_USER
+			 */
+			g_message("%s: COM_CHANGE_USER's auth-method-switch detected, but is currently not supported. Closing connection.",
+					G_STRLOC);
+			return -1;
 		default:
-			g_error("%s.%d: COM_(0x%02x) should be (ERR|OK), got %02x",
-					__FILE__, __LINE__,
-					con->parse.command, status);
-			break;
+			g_debug_hexdump(G_STRLOC, S(packet->data));
+			g_message("%s: got a 0x%02x packet as respose for COM_[0%02x], but expected only (ERR|OK)",
+					G_STRLOC,
+					con->parse.command,
+					(guint8)status);
+			return -1;
 		}
 		break;
 	case COM_INIT_DB:
@@ -586,10 +599,12 @@ int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld
 			is_finished = 1;
 			break;
 		default:
-			g_error("%s.%d: COM_(0x%02x) should be (ERR|OK), got 0x%02x",
-					__FILE__, __LINE__,
-					con->parse.command, (guint8)status);
-			break;
+			g_debug_hexdump(G_STRLOC, S(packet->data));
+			g_message("%s: got a 0x%02x packet as respose for COM_[0%02x], but expected only (ERR|OK)",
+					G_STRLOC,
+					con->parse.command,
+					(guint8)status);
+			return -1;
 		}
 		break;
 	case COM_DEBUG:
@@ -604,10 +619,12 @@ int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld
 			is_finished = 1;
 			break;
 		default:
-			g_error("%s.%d: COM_(0x%02x) should be EOF, got x%02x",
-					__FILE__, __LINE__,
-					con->parse.command, (guint8)status);
-			break;
+			g_debug_hexdump(G_STRLOC, S(packet->data));
+			g_message("%s: got a 0x%02x packet as respose for COM_[0%02x], but expected only (ERR|EOF)",
+					G_STRLOC,
+					con->parse.command,
+					(guint8)status);
+			return -1;
 		}
 		break;
 
@@ -621,13 +638,15 @@ int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld
 		case MYSQLD_PACKET_EOF:
 			is_finished = 1;
 			break;
+
 		case MYSQLD_PACKET_NULL:
 		case MYSQLD_PACKET_OK:
-			g_error("%s.%d: COM_(0x%02x) should not be (OK|ERR|NULL), got: %02x",
-					__FILE__, __LINE__,
-					con->parse.command, status);
-
-			break;
+			g_debug_hexdump(G_STRLOC, S(packet->data));
+			g_message("%s: got a 0x%02x packet as respose for COM_[0%02x], but expected only (ERR, EOF or field data)",
+					G_STRLOC,
+					con->parse.command,
+					(guint8)status);
+			return -1;
 		default:
 			break;
 		}
@@ -688,7 +707,8 @@ int network_mysqld_proto_get_query_result(network_packet *packet, network_mysqld
 		is_finished = 1;
 		break;
 	default:
-		g_critical("%s: COM_(0x%02x) is not handled", 
+		g_debug_hexdump(G_STRLOC, S(packet->data));
+		g_message("%s: COM_(0x%02x) is not handled", 
 				G_STRLOC,
 				con->parse.command);
 		err = 1;
@@ -1099,12 +1119,12 @@ network_mysqld_auth_challenge *network_mysqld_auth_challenge_new() {
 
 	shake = g_new0(network_mysqld_auth_challenge, 1);
 	
-	shake->challenge = g_string_new("");
+	shake->auth_plugin_data = g_string_new("");
 	shake->capabilities = 
 		CLIENT_PROTOCOL_41 |
 		CLIENT_SECURE_CONNECTION |
 		0;
-
+	shake->auth_plugin_name = g_string_new(NULL);
 
 	return shake;
 }
@@ -1113,31 +1133,54 @@ void network_mysqld_auth_challenge_free(network_mysqld_auth_challenge *shake) {
 	if (!shake) return;
 
 	if (shake->server_version_str) g_free(shake->server_version_str);
-	if (shake->challenge)          g_string_free(shake->challenge, TRUE);
+	if (shake->auth_plugin_data)   g_string_free(shake->auth_plugin_data, TRUE);
+	if (shake->auth_plugin_name)   g_string_free(shake->auth_plugin_name, TRUE);
 
 	g_free(shake);
 }
 
+network_mysqld_auth_challenge *network_mysqld_auth_challenge_copy(network_mysqld_auth_challenge *src) {
+	network_mysqld_auth_challenge *dst;
+
+	if (!src) return NULL;
+
+	dst = network_mysqld_auth_challenge_new();
+	dst->protocol_version = src->protocol_version;
+	dst->server_version  = src->server_version;
+	dst->thread_id       = src->thread_id;
+	dst->capabilities    = src->capabilities;
+	dst->charset         = src->charset;
+	dst->server_status   = src->server_status;
+	dst->server_version_str = g_strdup(src->server_version_str);
+	g_string_assign_len(dst->auth_plugin_data, S(src->auth_plugin_data));
+	g_string_assign_len(dst->auth_plugin_name, S(src->auth_plugin_name));
+
+	return dst;
+}
+
+
 void network_mysqld_auth_challenge_set_challenge(network_mysqld_auth_challenge *shake) {
 	guint i;
 
 	/* 20 chars */
 
-	g_string_set_size(shake->challenge, 21);
+	g_string_set_size(shake->auth_plugin_data, 21);
 
 	for (i = 0; i < 20; i++) {
-		shake->challenge->str[i] = (94.0 * (rand() / (RAND_MAX + 1.0))) + 33; /* 33 - 127 are printable characters */
+		shake->auth_plugin_data->str[i] = (94.0 * (rand() / (RAND_MAX + 1.0))) + 33; /* 33 - 127 are printable characters */
 	}
 
-	shake->challenge->len = 20;
-	shake->challenge->str[shake->challenge->len] = '\0';
+	shake->auth_plugin_data->len = 20;
+	shake->auth_plugin_data->str[shake->auth_plugin_data->len] = '\0';
 }
 
 int network_mysqld_proto_get_auth_challenge(network_packet *packet, network_mysqld_auth_challenge *shake) {
 	int maj, min, patch;
-	gchar *scramble_1 = NULL, *scramble_2 = NULL;
+	gchar *auth_plugin_data_1 = NULL, *auth_plugin_data_2 = NULL;
+	guint16 capabilities1, capabilities2;
 	guint8 status;
 	int err = 0;
+	guint8 auth_plugin_data_len;
 
 	err = err || network_mysqld_proto_get_int8(packet, &status);
 
@@ -1164,21 +1207,53 @@ int network_mysqld_proto_get_auth_challenge(network_packet *packet, network_mysq
 	/**
 	 * get the scramble buf
 	 *
-	 * 8 byte here and some the other 12 somewhen later
+	 * 8 byte here and some the other 12 sometime later
 	 */	
-	err = err || network_mysqld_proto_get_string_len(packet, &scramble_1, 8);
+	err = err || network_mysqld_proto_get_string_len(packet, &auth_plugin_data_1, 8);
 
 	err = err || network_mysqld_proto_skip(packet, 1);
 
-	err = err || network_mysqld_proto_get_int16(packet, &shake->capabilities);
+	err = err || network_mysqld_proto_get_int16(packet, &capabilities1);
 	err = err || network_mysqld_proto_get_int8(packet, &shake->charset);
 	err = err || network_mysqld_proto_get_int16(packet, &shake->server_status);
+
+	/* capabilities is extended in 5.5.x to carry 32bits to announce CLIENT_PLUGIN_AUTH */	
+	err = err || network_mysqld_proto_get_int16(packet, &capabilities2);
+	err = err || network_mysqld_proto_get_int8(packet, &auth_plugin_data_len);
+
+	err = err || network_mysqld_proto_skip(packet, 10);
+
+	if (!err) {
+		shake->capabilities = capabilities1 | (capabilities2 << 16);
 	
-	err = err || network_mysqld_proto_skip(packet, 13);
-	
-	if (shake->capabilities & CLIENT_SECURE_CONNECTION) {
-		err = err || network_mysqld_proto_get_string_len(packet, &scramble_2, 12);
-		err = err || network_mysqld_proto_skip(packet, 1);
+		if (shake->capabilities & CLIENT_PLUGIN_AUTH) {
+			guint8 auth_plugin_data2_len = 0;
+
+			/* CLIENT_PLUGIN_AUTH enforces auth_plugin_data_len
+			 *
+			 * we have at least 12 bytes */
+
+			if (auth_plugin_data_len > 8) {
+				auth_plugin_data2_len = auth_plugin_data_len - 8;
+			}
+
+			err = err || network_mysqld_proto_get_string_len(packet, &auth_plugin_data_2, auth_plugin_data2_len);
+			err = err || network_mysqld_proto_skip(packet, 12 - MIN(12, auth_plugin_data2_len));
+			if (!err) {
+				/* Bug#59453 ... MySQL 5.5.7-9 and 5.6.0-1 don't send a trailing \0
+				 *
+				 * if there is no trailing \0, get the rest of the packet
+				 */
+				if (0 != network_mysqld_proto_get_gstring(packet, shake->auth_plugin_name)) {
+					err = err || network_mysqld_proto_get_gstring_len(packet,
+							packet->data->len - packet->offset,
+							shake->auth_plugin_name);
+				}
+			}
+		} else if (shake->capabilities & CLIENT_SECURE_CONNECTION) {
+			err = err || network_mysqld_proto_get_string_len(packet, &auth_plugin_data_2, 12);
+			err = err || network_mysqld_proto_skip(packet, 1);
+		}
 	}
 
 	if (!err) {
@@ -1210,18 +1285,44 @@ int network_mysqld_proto_get_auth_challenge(network_packet *packet, network_mysq
 	
 	
 		/**
-		 * scramble_1 + scramble_2 == scramble
+		 * build auth_plugin_data
 		 *
-		 * a len-encoded string
+		 * auth_plugin_data_1 + auth_plugin_data_2 == auth_plugin_data
 		 */
-	
-		g_string_truncate(shake->challenge, 0);
-		g_string_append_len(shake->challenge, scramble_1, 8);
-		if (scramble_2) g_string_append_len(shake->challenge, scramble_2, 12); /* in old-password, no 2nd scramble */
+		g_string_truncate(shake->auth_plugin_data, 0);
+
+		if (shake->capabilities & CLIENT_PLUGIN_AUTH) {
+			g_string_assign_len(shake->auth_plugin_data, auth_plugin_data_1, MIN(8, auth_plugin_data_len));
+			if (auth_plugin_data_len > 8) {
+				g_string_append_len(shake->auth_plugin_data, auth_plugin_data_2, auth_plugin_data_len - 8);
+			}
+		} else if (shake->capabilities & CLIENT_SECURE_CONNECTION) {
+			g_string_assign_len(shake->auth_plugin_data, auth_plugin_data_1, 8);
+			g_string_append_len(shake->auth_plugin_data, auth_plugin_data_2, 12);
+		} else {
+			/* we have at least the old password scramble */
+			g_string_assign_len(shake->auth_plugin_data, auth_plugin_data_1, 8);
+		}
+
+		/* some final assertions */
+		if (shake->capabilities & CLIENT_PLUGIN_AUTH) {
+			if (shake->auth_plugin_data->len != auth_plugin_data_len) {
+				err = 1;
+			}
+		} else if (shake->capabilities & CLIENT_SECURE_CONNECTION) {
+			if (shake->auth_plugin_data->len != 20) {
+				err = 1;
+			}
+		} else {
+			/* old auth */
+			if (shake->auth_plugin_data->len != 8) {
+				err = 1;
+			}
+		}
 	}
 
-	if (scramble_1) g_free(scramble_1);
-	if (scramble_2) g_free(scramble_2);
+	if (auth_plugin_data_1) g_free(auth_plugin_data_1);
+	if (auth_plugin_data_2) g_free(auth_plugin_data_2);
 
 	return err ? -1 : 0;
 }
@@ -1243,31 +1344,54 @@ int network_mysqld_proto_append_auth_challenge(GString *packet, network_mysqld_a
 	}
 	network_mysqld_proto_append_int8(packet, 0x00);
 	network_mysqld_proto_append_int32(packet, shake->thread_id);
-	if (shake->challenge->len) {
-		g_string_append_len(packet, shake->challenge->str, 8);
+	if (shake->auth_plugin_data->len) {
+		g_assert_cmpint(shake->auth_plugin_data->len, >=, 8);
+		g_string_append_len(packet, shake->auth_plugin_data->str, 8);
 	} else {
 		g_string_append_len(packet, C("01234567"));
 	}
 	network_mysqld_proto_append_int8(packet, 0x00); /* filler */
-	network_mysqld_proto_append_int16(packet, shake->capabilities);
+	network_mysqld_proto_append_int16(packet, shake->capabilities & 0xffff);
 	network_mysqld_proto_append_int8(packet, shake->charset);
 	network_mysqld_proto_append_int16(packet, shake->server_status);
+	network_mysqld_proto_append_int16(packet, (shake->capabilities >> 16) & 0xffff);
 
-	for (i = 0; i < 13; i++) {
+	if (shake->capabilities & CLIENT_PLUGIN_AUTH) {
+		g_assert_cmpint(shake->auth_plugin_data->len, <, 255);
+		network_mysqld_proto_append_int8(packet, shake->auth_plugin_data->len);
+	} else {
+		network_mysqld_proto_append_int8(packet, 0);
+	}
+
+	/* add the fillers */
+	for (i = 0; i < 10; i++) {
 		network_mysqld_proto_append_int8(packet, 0x00);
 	}
 
-	if (shake->challenge->len) {
-		g_string_append_len(packet, shake->challenge->str + 8, 12);
-	} else {
-		g_string_append_len(packet, C("890123456789"));
+	if (shake->capabilities & CLIENT_PLUGIN_AUTH) {
+		g_assert_cmpint(shake->auth_plugin_data->len, >=, 8);
+		g_string_append_len(packet, shake->auth_plugin_data->str + 8, shake->auth_plugin_data->len - 8);
+
+		g_string_append_len(packet, S(shake->auth_plugin_name));
+		if ((shake->server_version >= 50510 && shake->server_version < 50600) ||
+		    (shake->server_version >= 50602)) {
+			g_string_append_c(packet, 0x00);
+		}
+	} else if (shake->capabilities & CLIENT_SECURE_CONNECTION) {
+		/* if we only have SECURE_CONNECTION it is 0-terminated */
+		if (shake->auth_plugin_data->len) {
+			g_assert_cmpint(shake->auth_plugin_data->len, >=, 8);
+			g_string_append_len(packet, shake->auth_plugin_data->str + 8, shake->auth_plugin_data->len - 8);
+		} else {
+			g_string_append_len(packet, C("890123456789"));
+		}
+		network_mysqld_proto_append_int8(packet, 0x00);
 	}
-	network_mysqld_proto_append_int8(packet, 0x00);
 	
 	return 0;
 }
 
-network_mysqld_auth_response *network_mysqld_auth_response_new() {
+network_mysqld_auth_response *network_mysqld_auth_response_new(guint32 server_capabilities) {
 	network_mysqld_auth_response *auth;
 
 	auth = g_new0(network_mysqld_auth_response, 1);
@@ -1275,10 +1399,12 @@ network_mysqld_auth_response *network_mysqld_auth_response_new() {
 	/* we have to make sure scramble->buf is not-NULL to get
 	 * the "empty string" and not a "NULL-string"
 	 */
-	auth->response = g_string_new("");
+	auth->auth_plugin_data = g_string_new("");
+	auth->auth_plugin_name = g_string_new(NULL);
 	auth->username = g_string_new("");
 	auth->database = g_string_new("");
-	auth->capabilities = CLIENT_SECURE_CONNECTION | CLIENT_PROTOCOL_41;
+	auth->client_capabilities = CLIENT_SECURE_CONNECTION | CLIENT_PROTOCOL_41;
+	auth->server_capabilities = server_capabilities;
 
 	return auth;
 }
@@ -1286,7 +1412,8 @@ network_mysqld_auth_response *network_mysqld_auth_response_new() {
 void network_mysqld_auth_response_free(network_mysqld_auth_response *auth) {
 	if (!auth) return;
 
-	if (auth->response)          g_string_free(auth->response, TRUE);
+	if (auth->auth_plugin_data)  g_string_free(auth->auth_plugin_data, TRUE);
+	if (auth->auth_plugin_name)  g_string_free(auth->auth_plugin_name, TRUE);
 	if (auth->username)          g_string_free(auth->username, TRUE);
 	if (auth->database)          g_string_free(auth->database, TRUE);
 
@@ -1326,39 +1453,45 @@ int network_mysqld_proto_get_auth_response(network_packet *packet, network_mysql
 	if (err) return -1;
 
 	if (l_cap & CLIENT_PROTOCOL_41) {
-		err = err || network_mysqld_proto_get_int32(packet, &auth->capabilities);
+		err = err || network_mysqld_proto_get_int32(packet, &auth->client_capabilities);
 		err = err || network_mysqld_proto_get_int32(packet, &auth->max_packet_size);
 		err = err || network_mysqld_proto_get_int8(packet, &auth->charset);
 
 		err = err || network_mysqld_proto_skip(packet, 23);
 	
 		err = err || network_mysqld_proto_get_gstring(packet, auth->username);
-		if (auth->capabilities & CLIENT_SECURE_CONNECTION) {
-			err = err || network_mysqld_proto_get_lenenc_gstring(packet, auth->response);
+		if ((auth->server_capabilities & CLIENT_SECURE_CONNECTION) &&
+		    (auth->server_capabilities & CLIENT_SECURE_CONNECTION)) {
+			guint8 len;
+			/* new auth is 1-byte-len + data */
+			err = err || network_mysqld_proto_get_int8(packet, &len);
+			err = err || network_mysqld_proto_get_gstring_len(packet, len, auth->auth_plugin_data);
 		} else {
-			err = err || network_mysqld_proto_get_gstring(packet, auth->response);
+			/* old auth stores it as NUL-term-string */
+			err = err || network_mysqld_proto_get_gstring(packet, auth->auth_plugin_data);
 		}
 
-		if (packet->offset != packet->data->len) {
-			/* database is optional and may include a trailing \0 char */
-			err = err || network_mysqld_proto_get_gstring_len(packet, packet->data->len - packet->offset, auth->database);
+		if ((auth->server_capabilities & CLIENT_CONNECT_WITH_DB) &&
+		    (auth->client_capabilities & CLIENT_CONNECT_WITH_DB)) {
+			err = err || network_mysqld_proto_get_gstring(packet, auth->database);
+		}
 
-			if (auth->database->len > 0 && 
-			    (auth->database->str[auth->database->len - 1] == '\0')) {
-				auth->database->len--;
-			}
+		if ((auth->server_capabilities & CLIENT_PLUGIN_AUTH) &&
+		    (auth->client_capabilities & CLIENT_PLUGIN_AUTH)) {
+			/* parse out the plugin name */
+			err = err || network_mysqld_proto_get_gstring(packet, auth->auth_plugin_name);
 		}
 	} else {
 		err = err || network_mysqld_proto_get_int16(packet, &l_cap);
 		err = err || network_mysqld_proto_get_int24(packet, &auth->max_packet_size);
 		err = err || network_mysqld_proto_get_gstring(packet, auth->username);
-		/* there may be no password sent */
 		if (packet->data->len != packet->offset) {
-			err = err || network_mysqld_proto_get_gstring(packet, auth->response);
+			/* if there is more, it is the password without a terminating \0 */
+			err = err || network_mysqld_proto_get_gstring_len(packet, packet->data->len - packet->offset, auth->auth_plugin_data);
 		}
 
 		if (!err) {
-			auth->capabilities = l_cap;
+			auth->client_capabilities = l_cap;
 		}
 	}
 
@@ -1371,19 +1504,18 @@ int network_mysqld_proto_get_auth_response(network_packet *packet, network_mysql
 int network_mysqld_proto_append_auth_response(GString *packet, network_mysqld_auth_response *auth) {
 	int i;
 
-	if (!(auth->capabilities & CLIENT_PROTOCOL_41)) {
-		network_mysqld_proto_append_int16(packet, auth->capabilities);
+	if (!(auth->client_capabilities & CLIENT_PROTOCOL_41)) {
+		network_mysqld_proto_append_int16(packet, auth->client_capabilities);
 		network_mysqld_proto_append_int24(packet, auth->max_packet_size); /* max-allowed-packet */
 
 		if (auth->username->len) g_string_append_len(packet, S(auth->username));
 		network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
 
-		if (auth->response->len) {
-			g_string_append_len(packet, S(auth->response));
-			network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
+		if (auth->auth_plugin_data->len) {
+			g_string_append_len(packet, S(auth->auth_plugin_data)); /* no trailing \0 */
 		}
 	} else {
-		network_mysqld_proto_append_int32(packet, auth->capabilities);
+		network_mysqld_proto_append_int32(packet, auth->client_capabilities);
 		network_mysqld_proto_append_int32(packet, auth->max_packet_size); /* max-allowed-packet */
 		
 		network_mysqld_proto_append_int8(packet, auth->charset); /* charset */
@@ -1396,11 +1528,31 @@ int network_mysqld_proto_append_auth_response(GString *packet, network_mysqld_au
 		network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
 
 		/* scrambled password */
-		network_mysqld_proto_append_lenenc_string_len(packet, S(auth->response));
-		if (auth->database->len) {
+		if (auth->server_capabilities & CLIENT_SECURE_CONNECTION) {
+			/* server supports the secure-auth (4.1+) which is 255 bytes max
+			 *
+			 * if ->len is longer than 255, wrap around ... should be reported back
+			 * to the upper layers
+			 */
+			network_mysqld_proto_append_int8(packet, auth->auth_plugin_data->len);
+			g_string_append_len(packet, auth->auth_plugin_data->str, auth->auth_plugin_data->len & 0xff);
+		} else {
+			/* server only supports the old protocol which allows any length, but no \0 in the auth-plugin-data */
+			g_string_append_len(packet, auth->auth_plugin_data->str, auth->auth_plugin_data->len);
+			network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
+		}
+
+		if ((auth->server_capabilities & CLIENT_CONNECT_WITH_DB) &&
+		    (auth->database->len > 0)) {
 			g_string_append_len(packet, S(auth->database));
 			network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
 		}
+
+		if ((auth->client_capabilities & CLIENT_PLUGIN_AUTH) &&
+		    (auth->server_capabilities & CLIENT_PLUGIN_AUTH)) {
+			g_string_append_len(packet, S(auth->auth_plugin_name));
+			network_mysqld_proto_append_int8(packet, 0x00); /* trailing \0 */
+		}
 	}
 
 	return 0;
@@ -1412,12 +1564,13 @@ network_mysqld_auth_response *network_mysqld_auth_response_copy(network_mysqld_a
 
 	if (!src) return NULL;
 
-	dst = network_mysqld_auth_response_new();
-	dst->capabilities    = src->capabilities;
+	dst = network_mysqld_auth_response_new(src->server_capabilities);
+	dst->client_capabilities    = src->client_capabilities;
 	dst->max_packet_size = src->max_packet_size;
 	dst->charset         = src->charset;
 	g_string_assign_len(dst->username, S(src->username));
-	g_string_assign_len(dst->response, S(src->response));
+	g_string_assign_len(dst->auth_plugin_data, S(src->auth_plugin_data));
+	g_string_assign_len(dst->auth_plugin_name, S(src->auth_plugin_name));
 	g_string_assign_len(dst->database, S(src->database));
 
 	return dst;
