@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2007, 2009, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -354,6 +354,25 @@ int network_mysqld_proto_get_int32(network_packet *packet, guint32 *v) {
 	return 0;
 }
 
+/**
+ * peek a 32-bit integer from the network packet
+ *
+ * @param packet the MySQL network packet
+ * @param v      dest for the number
+ * @return 0 on success, non-0 on error
+ * @see network_mysqld_proto_peek_int_len()
+ */
+int network_mysqld_proto_peek_int32(network_packet *packet, guint32 *v) {
+	guint64 v64;
+
+	if (network_mysqld_proto_peek_int_len(packet, &v64, 4)) return -1;
+
+	*v = v64 & 0xffffffff;
+
+	return 0;
+}
+
+
 /**
  * get a 6-byte integer from the network packet
  *
@@ -1093,21 +1112,6 @@ gboolean network_mysqld_proto_password_check(
 	return is_same;
 }
 
-
-network_packet *network_packet_new(void) {
-	network_packet *packet;
-
-	packet = g_new0(network_packet, 1);
-
-	return packet;
-}
-
-void network_packet_free(network_packet *packet) {
-	if (!packet) return;
-
-	g_free(packet);
-}
-
 int network_mysqld_proto_skip_network_header(network_packet *packet) {
 	return network_mysqld_proto_skip(packet, NET_HEADER_SIZE);
 }
