@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -72,6 +72,8 @@
 #include "chassis-event-thread.h"
 #include "lua-scope.h"
 #include "glib-ext.h"
+#include "network-asn1.h"
+#include "network-spnego.h"
 
 #if defined(HAVE_SYS_SDT_H) && defined(ENABLE_DTRACE)
 #include <sys/sdt.h>
@@ -137,6 +139,39 @@ network_socket_retval_t plugin_call_cleanup(chassis *srv, network_mysqld_con *co
 	return retval;
 }
 
+/**
+ * call the timeout callback for the current connection
+ *
+ * @param srv    global context
+ * @param con    connection context
+ *
+ * @return       NETWORK_SOCKET_SUCCESS on success
+ */
+static network_socket_retval_t
+plugin_call_timeout(chassis *srv, network_mysqld_con *con) {
+	NETWORK_MYSQLD_PLUGIN_FUNC(func) = NULL;
+	network_socket_retval_t retval = NETWORK_SOCKET_ERROR;
+
+	func = con->plugins.con_timeout;
+	
+	if (!func) {
+		/* default implementation */
+		g_debug("%s: connection between %s and %s timed out. closing it",
+				G_STRLOC,
+				con->client ? con->client->src->name->str : "(client)",
+				con->server ? con->server->dst->name->str : "(server)");
+		con->state = CON_STATE_ERROR;
+		return NETWORK_SOCKET_SUCCESS;
+	}
+
+	LOCK_LUA(srv->priv->sc);
+	retval = (*func)(srv, con);
+	UNLOCK_LUA(srv->priv->sc);
+
+	return retval;
+}
+
+
 chassis_private *network_mysqld_priv_init(void) {
 	chassis_private *priv;
 
@@ -205,6 +240,25 @@ network_mysqld_con *network_mysqld_con_new() {
 	con = g_new0(network_mysqld_con, 1);
 	con->timestamps = chassis_timestamps_new();
 	con->parse.command = -1;
+	con->auth_switch_to_method = g_string_new(NULL);
+	con->auth_switch_to_round  = 0;
+	con->auth_switch_to_data   = g_string_new(NULL);;
+
+	/* some tiny helper macros */
+#define SECONDS ( 1 )
+#define MINUTES ( 60 * SECONDS )
+#define HOURS ( 60 * MINUTES )
+	con->connect_timeout.tv_sec = 2 * SECONDS;
+	con->connect_timeout.tv_usec = 0;
+
+	con->read_timeout.tv_sec = 8 * HOURS;
+	con->read_timeout.tv_usec = 0;
+	
+	con->write_timeout.tv_sec = 8 * HOURS;
+	con->write_timeout.tv_usec = 0;
+#undef SECONDS
+#undef MINUTES
+#undef HOURS
 
 	return con;
 }
@@ -232,6 +286,9 @@ void network_mysqld_con_free(network_mysqld_con *con) {
 	if (con->server) network_socket_free(con->server);
 	if (con->client) network_socket_free(con->client);
 
+	g_string_free(con->auth_switch_to_method, TRUE);
+	g_string_free(con->auth_switch_to_data, TRUE);
+
 	/* we are still in the conns-array */
 
 	g_ptr_array_remove_fast(con->srv->priv->cons, con);
@@ -653,65 +710,122 @@ network_socket_retval_t plugin_call(chassis *srv, network_mysqld_con *con, int s
 	case CON_STATE_READ_AUTH_RESULT:
 		func = con->plugins.con_read_auth_result;
 		break;
-	case CON_STATE_SEND_AUTH_RESULT:
+	case CON_STATE_SEND_AUTH_RESULT: 
+		/* called after the auth data is sent to the client */
 		func = con->plugins.con_send_auth_result;
 
-		if (!func) { /* default implementation */
+		if (!func) {
+			/*
+			 * figure out what to do next:
+			 * - switch to 'read command from client'
+			 * - close connection
+			 * - read auth-data from client
+			 * - read another auth-result packet from server
+			 */
 			switch (con->auth_result_state) {
 			case MYSQLD_PACKET_OK:
+				/* OK, delivered to client, switch to command phase */
 				con->state = CON_STATE_READ_QUERY;
 				break;
 			case MYSQLD_PACKET_ERR:
+				/* ERR delivered to client, close the connection now */
 				con->state = CON_STATE_ERROR;
 				break;
-			case MYSQLD_PACKET_EOF:
+			case 0x01: /* more auth data */
 				/**
-				 * the MySQL 4.0 hash in a MySQL 4.1+ connection
+				 * FIXME: we should track that the server only sends us a 0x01 reply if
+				 * we first went through "switch auth packet"
+				 */
+
+				/**
+				 * if we switched to win-auth and SPNEGO is used, check if the response packet contains:
+				 * 
+				 *   negState = accept-succeeded.
+				 */
+				if ((strleq(S(con->auth_switch_to_method), C("authentication_windows_client"))) &&
+				    con->auth_next_packet_is_from_server) {
+					/* we either have SPNEGO or NTLM */
+					con->state = CON_STATE_READ_AUTH_RESULT;
+					break;
+				}
+			case MYSQLD_PACKET_EOF:
+				/*
+				 * next, read the auth data from the client
 				 */
 				con->state = CON_STATE_READ_AUTH_OLD_PASSWORD;
 				break;
 			default:
-				g_error("%s.%d: unexpected state for SEND_AUTH_RESULT: %02x", 
+				g_debug("%s.%d: unexpected state for SEND_AUTH_RESULT: %02x", 
 						__FILE__, __LINE__,
 						con->auth_result_state);
+				con->state = CON_STATE_ERROR;
+				break;
 			}
 		}
 		break;
-	case CON_STATE_READ_AUTH_OLD_PASSWORD: {
-		/** move the packet to the send queue */
-		GString *packet;
-		GList *chunk;
-		network_socket *recv_sock, *send_sock;
+	case CON_STATE_READ_AUTH_OLD_PASSWORD:
+		func = con->plugins.con_read_auth_old_password;
 
-		recv_sock = con->client;
-		send_sock = con->server;
+		if (!func) {
+			network_socket *recv_sock, *send_sock;
+			network_packet packet;
+			guint32 packet_len;
 
-		if (NULL == con->server) {
-			/**
-			 * we have to auth against same backend as we did before
-			 * but the user changed it
-			 */
+			/* move the packet to the send queue */
 
-			g_message("%s.%d: (lua) read-auth-old-password failed as backend_ndx got reset.", __FILE__, __LINE__);
+			recv_sock = con->client;
+			send_sock = con->server;
 
-			network_mysqld_con_send_error(con->client, C("(lua) read-auth-old-password failed as backend_ndx got reset."));
-			con->state = CON_STATE_SEND_ERROR;
-			break;
+			if (NULL == con->server) {
+				/**
+				 * we have to auth against same backend as we did before
+				 * but the user changed it
+				 */
+
+				g_message("%s.%d: (lua) read-auth-old-password failed as backend_ndx got reset.", __FILE__, __LINE__);
+
+				network_mysqld_con_send_error(con->client, C("(lua) read-auth-old-password failed as backend_ndx got reset."));
+				con->state = CON_STATE_SEND_ERROR;
+				break;
+			}
+
+			packet.data = g_queue_peek_head(recv_sock->recv_queue->chunks);
+			packet.offset = 0;
+
+			packet_len = network_mysqld_proto_get_packet_len(packet.data);
+
+			if ((strleq(S(con->auth_switch_to_method), C("authentication_windows_client"))) &&
+			    (con->auth_switch_to_round == 0) &&
+			    (packet_len == 255)) {
+#if 1
+				/**
+				 * FIXME: the 2-packet win-auth protocol enhancements aren't properly tested yet.
+				 * therefore they are disabled for now.
+				 */
+				g_string_free(g_queue_pop_head(recv_sock->recv_queue->chunks), TRUE);
+
+				network_mysqld_con_send_error(recv_sock, C("long packets for windows-authentication aren't completely handled yet. Please use another auth-method for now."));
+
+				con->state = CON_STATE_SEND_ERROR;
+#else
+				con->auth_switch_to_round++;
+				/* move the packet to the send-queue
+				 */
+				network_mysqld_queue_append_raw(send_sock, send_sock->send_queue,
+						g_queue_pop_head(recv_sock->recv_queue->chunks));
+
+				/* stay in this state and read the next packet too */
+#endif
+			} else {
+				/* move the packet to the send-queue
+				 */
+				network_mysqld_queue_append_raw(send_sock, send_sock->send_queue,
+						g_queue_pop_head(recv_sock->recv_queue->chunks));
+
+				con->state = CON_STATE_SEND_AUTH_OLD_PASSWORD;
+			}
 		}
-
-		chunk = recv_sock->recv_queue->chunks->head;
-		packet = chunk->data;
-
-		/* we aren't finished yet */
-		network_queue_append(send_sock->send_queue, packet);
-
-		g_queue_delete_link(recv_sock->recv_queue->chunks, chunk);
-
-		/**
-		 * send it out to the client 
-		 */
-		con->state = CON_STATE_SEND_AUTH_OLD_PASSWORD;
-		break; }
+		break;
 	case CON_STATE_SEND_AUTH_OLD_PASSWORD:
 		/**
 		 * data is at the server, read the response next 
@@ -829,6 +943,77 @@ const char *network_mysqld_con_state_get_name(network_mysqld_con_state_t state)
 	return "unknown";
 }
 
+static int network_mysqld_con_track_auth_result_state(network_mysqld_con *con) {
+	network_packet packet;
+	guint8 state;
+	int err = 0;
+
+	/**
+	 * depending on the result-set we have different exit-points
+	 * - OK  -> READ_QUERY
+	 * - EOF -> (read old password hash) 
+	 * - ERR -> ERROR
+	 */
+	packet.data = g_queue_peek_head(con->server->recv_queue->chunks);
+	packet.offset = 0;
+
+	err = err || network_mysqld_proto_skip_network_header(&packet);
+	err = err || network_mysqld_proto_peek_int8(&packet, &state);
+
+	if (err) return -1;
+
+	con->auth_result_state = state;
+
+	if (state == 0xfe) {
+		/* a long auth-switch packet */
+		err = err || network_mysqld_proto_skip(&packet, 1);
+
+		if (packet.data->len - packet.offset > 0) {
+			err = err || network_mysqld_proto_get_gstring(&packet, con->auth_switch_to_method);
+			err = err || network_mysqld_proto_get_gstring_len(&packet, packet.data->len - packet.offset, con->auth_switch_to_data);
+		} else {
+			/* just in case we get here switch ... which shouldn't happen ... */
+			g_string_truncate(con->auth_switch_to_method, 0);
+			g_string_truncate(con->auth_switch_to_data, 0);
+		}
+		con->auth_switch_to_round = 0;
+		con->auth_next_packet_is_from_server = FALSE;
+	} else if (state == 0x01) {
+		if ((strleq(S(con->auth_switch_to_method), C("authentication_windows_client")))) {
+			GError *gerr = NULL;
+
+			/* if the packet comes from the server, has a 0x01, is SPNEGO and has 'accept-completed' set,
+			 * the next packet comes from the server too 
+			 */
+			if (0 != network_mysqld_proto_skip(&packet, 1)) {
+				/* hmm ... what to do now ? */
+				err = 1;
+			} else if (FALSE == network_asn1_is_valid(&packet, &gerr)) {
+				g_debug("%s: ASN1 packet is invalid: %s", G_STRLOC, gerr->message);
+				g_clear_error(&gerr);
+				err = 1;
+			} else {
+				network_spnego_response_token *token;
+
+				token = network_spnego_response_token_new();
+
+				if (TRUE == network_spnego_proto_get_response_token(&packet, token, &gerr)) {
+					if (token->negState != SPNEGO_RESPONSE_STATE_ACCEPT_INCOMPLETE) {
+						con->auth_next_packet_is_from_server = TRUE;
+					}
+				} else {
+					g_debug("%s: parsing spnego failed: %s", G_STRLOC, gerr->message);
+					/* do we care why it failed ? */
+					g_clear_error(&gerr);
+				}
+
+				network_spnego_response_token_free(token);
+			}
+		}
+	}
+	return err ? -1 : 0;
+}
+
 /**
  * handle the different states of the MySQL protocol
  *
@@ -896,11 +1081,21 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				con->state = CON_STATE_ERROR;
 			}
 		}
+	} else if (events == EV_TIMEOUT) {
+		/* if we got a timeout on CON_STATE_CONNECT_SERVER we should pick another backend */
+		switch ((retval = plugin_call_timeout(srv, con))) {
+		case NETWORK_SOCKET_SUCCESS:
+			/* the plugin did set a reasonable next state */
+			break;
+		default:
+			con->state = CON_STATE_ERROR;
+			break;
+		}
 	}
 
 #define WAIT_FOR_EVENT(ev_struct, ev_type, timeout) \
 	event_set(&(ev_struct->event), ev_struct->fd, ev_type, network_mysqld_con_handle, user_data); \
-	chassis_event_add(srv, &(ev_struct->event)); 
+	chassis_event_add_with_timeout(srv, &(ev_struct->event), timeout); 
 
 	/**
 	 * loop on the same connection as long as we don't end up in a stable state
@@ -913,6 +1108,8 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 	}
 
 	do {
+		struct timeval timeout;
+
 		ostate = con->state;
 #ifdef NETWORK_DEBUG_TRACE_STATE_CHANGES
 		/* if you need the state-change information without dtrace, enable this */
@@ -1036,10 +1233,11 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			case NETWORK_SOCKET_ERROR_RETRY:
 				if (con->server) {
+					timeout = con->connect_timeout;
 					/**
 					 * we have a server connection waiting to begin writable
 					 */
-					WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+					WAIT_FOR_EVENT(con->server, EV_WRITE, &timeout);
 					NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::connect_server");
 					return;
 				} else {
@@ -1081,9 +1279,11 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			switch (network_mysqld_read(srv, recv_sock)) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
-			case NETWORK_SOCKET_WAIT_FOR_EVENT:
+			case NETWORK_SOCKET_WAIT_FOR_EVENT: 
+				timeout = con->read_timeout;
+
 				/* call us again when you have a event */
-				WAIT_FOR_EVENT(con->server, EV_READ, NULL);
+				WAIT_FOR_EVENT(con->server, EV_READ, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_handshake");
 
 				return;
@@ -1123,8 +1323,10 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			switch (network_mysqld_write(srv, con->client)) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
-			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+			case NETWORK_SOCKET_WAIT_FOR_EVENT: 
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->client, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_handshake");
 				
 				return;
@@ -1161,7 +1363,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->client, EV_READ, NULL);
+				timeout = con->read_timeout;
+
+				WAIT_FOR_EVENT(con->client, EV_READ, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_auth");
 
 				return;
@@ -1193,7 +1397,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->server, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_auth");
 
 				return;
@@ -1220,8 +1426,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 		case CON_STATE_READ_AUTH_RESULT: {
 			/* read the auth result from the server */
 			network_socket *recv_sock;
-			GList *chunk;
-			GString *packet;
+
 			recv_sock = con->server;
 
 			g_assert(events == 0 || event_fd == recv_sock->fd);
@@ -1230,7 +1435,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->server, EV_READ, NULL);
+				timeout = con->read_timeout;
+
+				WAIT_FOR_EVENT(con->server, EV_READ, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_auth_result");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1241,18 +1448,10 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			}
 			if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
 
-			/**
-			 * depending on the result-set we have different exit-points
-			 * - OK  -> READ_QUERY
-			 * - EOF -> (read old password hash) 
-			 * - ERR -> ERROR
-			 */
-			chunk = recv_sock->recv_queue->chunks->head;
-			packet = chunk->data;
-			g_assert(packet);
-			g_assert(packet->len > NET_HEADER_SIZE);
-
-			con->auth_result_state = packet->str[NET_HEADER_SIZE];
+			if (0 != network_mysqld_con_track_auth_result_state(con)) {
+				con->state = CON_STATE_ERROR;
+				break;
+			}
 
 			switch (plugin_call(srv, con, con->state)) {
 			case NETWORK_SOCKET_SUCCESS:
@@ -1272,7 +1471,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->client, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_auth_result");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1301,7 +1502,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->client, EV_READ, NULL);
+				timeout = con->read_timeout;
+
+				WAIT_FOR_EVENT(con->client, EV_READ, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_auth_old_password");
 
 				return;
@@ -1330,7 +1533,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->server, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_auth_old_password");
 
 				return;
@@ -1367,7 +1572,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				case NETWORK_SOCKET_SUCCESS:
 					break;
 				case NETWORK_SOCKET_WAIT_FOR_EVENT:
-					WAIT_FOR_EVENT(con->client, EV_READ, NULL);
+					timeout = con->read_timeout;
+
+					WAIT_FOR_EVENT(con->client, EV_READ, &timeout);
 					NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_query");
 					return;
 				case NETWORK_SOCKET_ERROR_RETRY:
@@ -1463,7 +1670,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->server, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_query");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1509,7 +1718,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				case NETWORK_SOCKET_SUCCESS:
 					break;
 				case NETWORK_SOCKET_WAIT_FOR_EVENT:
-					WAIT_FOR_EVENT(con->server, EV_READ, NULL);
+					timeout = con->read_timeout;
+
+					WAIT_FOR_EVENT(con->server, EV_READ, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_query_result");
 					return;
 				case NETWORK_SOCKET_ERROR_RETRY:
@@ -1551,7 +1762,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->client, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_query_result");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1607,8 +1820,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				case NETWORK_SOCKET_SUCCESS:
 					break;
 				case NETWORK_SOCKET_WAIT_FOR_EVENT:
+					timeout = con->read_timeout;
 					/* call us again when you have a event */
-					WAIT_FOR_EVENT(recv_sock, EV_READ, NULL);
+					WAIT_FOR_EVENT(recv_sock, EV_READ, &timeout);
 					NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_load_infile_data");
 
 					return;
@@ -1623,8 +1837,20 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 
 				if (con->state != ostate) break; /* the state has changed (e.g. CON_STATE_ERROR) */
 
+				/**
+				 * do the plugin call to decode the result-set to track if we are finished already
+				 * or we need to keep reading the data
+				 */
 				switch ((call_ret = plugin_call(srv, con, con->state))) {
 				case NETWORK_SOCKET_SUCCESS:
+					/**
+					 * if we still haven't read all data from LDLI, lets forward immediatly
+					 * the data to the backends so that we can read the next packets
+					 */
+					if (!con->local_file_data_is_finished && con->server) {
+						con->state = CON_STATE_SEND_LOCAL_INFILE_DATA;
+					}
+
 					break;
 				default:
 					g_critical("%s: plugin_call(%s) unexpected return value: %d",
@@ -1635,7 +1861,8 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 					con->state = CON_STATE_ERROR;
 					break;
 				}
-			} while (con->state == ostate); /* read packets from the network until the plugin decodes to go to the next state */
+			/* read packets from the network until the plugin decodes to go to the next state */
+			} while (con->state == CON_STATE_READ_LOCAL_INFILE_DATA);
 	
 			break; }
 		case CON_STATE_SEND_LOCAL_INFILE_DATA: 
@@ -1643,9 +1870,22 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 
 			switch (network_mysqld_write(srv, con->server)) {
 			case NETWORK_SOCKET_SUCCESS:
+				/* if we still haven't read all data from LDLI so we need to go back and read more
+				 */
+				if (!con->local_file_data_is_finished && con->server) {
+					con->state = CON_STATE_READ_LOCAL_INFILE_DATA;
+				}
+				/* we have read all data from LDLI so we need to read the LDLI result from the server
+				 */
+				else {
+					con->state = CON_STATE_READ_LOCAL_INFILE_RESULT;
+				}
+
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->server, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_load_infile_data");
 				
 				return;
@@ -1686,8 +1926,10 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
+				timeout = con->read_timeout;
+
 				/* call us again when you have a event */
-				WAIT_FOR_EVENT(recv_sock, EV_READ, NULL);
+				WAIT_FOR_EVENT(recv_sock, EV_READ, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::read_load_infile_result");
 
 				return;
@@ -1724,7 +1966,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->client, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_load_infile_result");
 				
 				return;
@@ -1762,7 +2006,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case NETWORK_SOCKET_SUCCESS:
 				break;
 			case NETWORK_SOCKET_WAIT_FOR_EVENT:
-				WAIT_FOR_EVENT(con->client, EV_WRITE, NULL);
+				timeout = con->write_timeout;
+
+				WAIT_FOR_EVENT(con->client, EV_WRITE, &timeout);
 				NETWORK_MYSQLD_CON_TRACK_TIME(con, "wait_for_event::send_error");
 				return;
 			case NETWORK_SOCKET_ERROR_RETRY:
@@ -1773,7 +2019,7 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				break;
 			}
 				
-			con->state = CON_STATE_ERROR;
+			con->state = CON_STATE_CLOSE_CLIENT;
 
 			break;
 		}
@@ -1782,6 +2028,20 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 		events   = 0;
 	} while (ostate != con->state);
 	NETWORK_MYSQLD_CON_TRACK_TIME(con, "con_handle_end");
+#if 0
+	/**
+	 * there are two ways to leave the state-engine:
+	 * - wait for event and return (WAIT_FOR_EVENT(); return;)
+	 * - close the socket and return
+	 *
+	 * if you have to stay in the same state create your own tiny loop
+	 *
+	 * in other words: this code below should never be triggered.
+	 */
+	g_critical("%s: left the MySQL protocol's state-machine at state '%s'. You may see the connection hang now.",
+			G_STRLOC,
+			network_mysqld_con_state_get_name(con->state));
+#endif
 
 	return;
 }
