@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2008, 2009, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -56,12 +56,15 @@ static int proxy_socket_get(lua_State *L) {
 			lua_pushlstring(L, S(sock->response->username));
 			return 1;
 		} else if (strleq(key, keysize, C("scrambled_password"))) {
-			lua_pushlstring(L, S(sock->response->response));
+			lua_pushlstring(L, S(sock->response->auth_plugin_data));
+			return 1;
+		} else if (strleq(key, keysize, C("auth_plugin_name"))) {
+			lua_pushlstring(L, S(sock->response->auth_plugin_name));
 			return 1;
 		}
 	}
 
-	if (sock->challenge) { /* only the server-side has mysqld_version set */
+	if (sock->challenge) {
 		if (strleq(key, keysize, C("mysqld_version"))) {
 			lua_pushinteger(L, sock->challenge->server_version);
 			return 1;
@@ -69,7 +72,10 @@ static int proxy_socket_get(lua_State *L) {
 			lua_pushinteger(L, sock->challenge->thread_id);
 			return 1;
 		} else if (strleq(key, keysize, C("scramble_buffer"))) {
-			lua_pushlstring(L, S(sock->challenge->challenge));
+			lua_pushlstring(L, S(sock->challenge->auth_plugin_data));
+			return 1;
+		} else if (strleq(key, keysize, C("auth_plugin_name"))) {
+			lua_pushlstring(L, S(sock->challenge->auth_plugin_name));
 			return 1;
 		}
 	}
