@@ -1,5 +1,5 @@
 --[[ $%BEGINLICENSE%$
- Copyright (c) 2008, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -170,6 +170,33 @@ function read_query(packet)
 			"\254\0\0\002\0"  -- no data EOF
 		}
 		
+		return proxy.PROXY_SEND_RESULT
+	elseif query == 'SELECT NULL' then
+		-- return a empty row
+		--
+		-- HINT: lua uses \ddd (3 decimal digits) instead of octals
+		proxy.response.type = proxy.MYSQLD_PACKET_RAW
+		proxy.response.packets = {
+			"\001",  -- one field
+			"\003def" ..   -- catalog
+			  "\0" ..    -- db 
+			  "\0" ..    -- table
+			  "\0" ..    -- orig-table
+			  "\0011" .. -- name
+			  "\0" ..    -- orig-name
+			  "\f" ..    -- filler
+			  "\008\0" .. -- charset
+			  " \0\0\0" .. -- length
+			  "\003" ..    -- type
+			  "\002\0" ..  -- flags 
+			  "\0" ..    -- decimals
+			  "\0\0",    -- filler
+
+			"\254\0\0\002\0", -- EOF
+			"\251", -- NULL
+			"\254\0\0\002\0"  -- no data EOF
+		}
+		
 		return proxy.PROXY_SEND_RESULT
 
 	else
