@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2008, 2009, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -34,13 +34,21 @@
 #define C(x) x, sizeof(x) - 1
 #define S(x) x->str, x->len
 
+#ifndef CLIENT_PLUGIN_AUTH
+#define CLIENT_PLUGIN_AUTH (1UL << 19)
+#endif
+
+#ifndef CLIENT_PS_MULTI_RESULTS
+#define CLIENT_PS_MULTI_RESULTS (1UL << 18)
+#endif
+
 /**
  * Tests for the MySQL Protocol Codec functions
  * @ingroup proto
  */
 
 /*@{*/
-void t_ok_packet_new(void) {
+static void t_ok_packet_new(void) {
 	network_mysqld_ok_packet_t *ok_packet;
 
 	ok_packet = network_mysqld_ok_packet_new();
@@ -49,7 +57,7 @@ void t_ok_packet_new(void) {
 	network_mysqld_ok_packet_free(ok_packet);
 }
 
-void t_ok_packet_append(void) {
+static void t_ok_packet_append(void) {
 	network_mysqld_ok_packet_t *ok_packet;
 	network_packet *packet;
 
@@ -100,7 +108,7 @@ void t_ok_packet_append(void) {
 	network_packet_free(packet);
 }
 
-void t_err_packet_new(void) {
+static void t_err_packet_new(void) {
 	network_mysqld_err_packet_t *err_packet;
 
 	err_packet = network_mysqld_err_packet_new();
@@ -109,7 +117,7 @@ void t_err_packet_new(void) {
 	network_mysqld_err_packet_free(err_packet);
 }
 
-void t_err_packet_append(void) {
+static void t_err_packet_append(void) {
 	network_mysqld_err_packet_t *err_packet;
 	network_packet *packet;
 
@@ -158,7 +166,7 @@ void t_err_packet_append(void) {
 	network_packet_free(packet);
 }
 
-void t_eof_packet_new(void) {
+static void t_eof_packet_new(void) {
 	network_mysqld_eof_packet_t *eof_packet;
 
 	eof_packet = network_mysqld_eof_packet_new();
@@ -167,7 +175,7 @@ void t_eof_packet_new(void) {
 	network_mysqld_eof_packet_free(eof_packet);
 }
 
-void t_eof_packet_append(void) {
+static void t_eof_packet_append(void) {
 	network_mysqld_eof_packet_t *eof_packet;
 	network_packet *packet;
 
@@ -214,7 +222,7 @@ void t_eof_packet_append(void) {
 	network_packet_free(packet);
 }
 
-void test_mysqld_handshake(void) {
+static void test_mysqld_handshake(void) {
 	const char raw_packet[] = "J\0\0\0"
 		"\n"
 		"5.0.45-Debian_1ubuntu3.3-log\0"
@@ -257,8 +265,8 @@ void test_mysqld_handshake(void) {
 			CLIENT_TRANSACTIONS |
 			CLIENT_SECURE_CONNECTION));
 
-	g_assert(shake->challenge->len == 20);
-	g_assert(0 == memcmp(shake->challenge->str, "\"L;!3|8@vV,s#PLjSA+Q", shake->challenge->len));
+	g_assert(shake->auth_plugin_data->len == 20);
+	g_assert(0 == memcmp(shake->auth_plugin_data->str, "\"L;!3|8@vV,s#PLjSA+Q", shake->auth_plugin_data->len));
 
 	/* ... and back */
 	g_string_truncate(packet.data, 0);
@@ -273,16 +281,157 @@ void test_mysqld_handshake(void) {
 	g_string_free(packet.data, TRUE);
 }
 
-void test_mysqld_auth_empty_pw(void) {
+static void test_mysqld_handshake_plugin_auth(void) {
+	const char raw_packet[] = 
+		"\x50\x00\x00\x00"
+		"\x0a"
+		"\x35\x2e\x36\x2e\x32\x2d\x6d\x35\x2d\x6c\x6f\x67\x00"
+		"\x4d\x00\x00\x00"
+		"\x3d\x25\x3d\x43\x76\x4d\x5e\x6d\x00"
+		"\xff\xff" /* capabilities - part 1 */
+		"\x08"     /* charset */
+		"\x02\x00" /* status */
+		"\x0f\xc0" /* capabilities - part 2 */
+		"\x15" /* auth-plugin-part-len */
+		"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" /* fillers */
+		"\x2d\x37\x69\x2d\x41\x3f\x58\x7c\x2c\x5f\x7d\x75\x00" /* auth-plugin-part-2 */
+		"\x6d\x79\x73\x71\x6c\x5f\x6e\x61\x74\x69\x76\x65\x5f\x70\x61\x73\x73\x77\x6f\x72\x64\x00" /* mysql_native_password\0 */
+		;
+
+	network_mysqld_auth_challenge *shake;
+	network_packet packet;
+
+	shake = network_mysqld_auth_challenge_new();
+	
+	packet.data = g_string_new(NULL);
+	packet.offset = 0;
+	g_string_append_len(packet.data, C(raw_packet));
+
+	g_assert_cmpint(packet.data->len, ==, 84);
+
+	g_assert_cmpint(0, ==, network_mysqld_proto_skip_network_header(&packet));
+	g_assert_cmpint(0, ==, network_mysqld_proto_get_auth_challenge(&packet, shake));
+
+	g_assert_cmpint(shake->server_version, ==, 50602);
+	g_assert_cmpint(shake->thread_id, ==, 77);
+	g_assert_cmpint(shake->server_status, ==,
+			SERVER_STATUS_AUTOCOMMIT);
+	g_assert_cmpint(shake->charset, ==, 8);
+	g_assert_cmphex(shake->capabilities, &,
+			(CLIENT_CONNECT_WITH_DB |
+			CLIENT_LONG_FLAG |
+
+			CLIENT_COMPRESS |
+
+			CLIENT_PROTOCOL_41 |
+
+			CLIENT_TRANSACTIONS |
+			CLIENT_SECURE_CONNECTION |
+			CLIENT_PLUGIN_AUTH));
+
+	g_assert_cmpint(shake->auth_plugin_data->len, ==, 21);
+	g_assert_cmpint(0, == ,memcmp(shake->auth_plugin_data->str, "=%=CvM^m-7i-A?X|,_}u\0", shake->auth_plugin_data->len));
+	g_assert_cmpstr(shake->auth_plugin_name->str, ==, "mysql_native_password");
+
+	/* ... and back */
+	g_string_truncate(packet.data, 0);
+	g_string_append_len(packet.data, C("P\0\0\0")); /* prepend length and sequence-id */
+	network_mysqld_proto_append_auth_challenge(packet.data, shake);
+
+	g_assert_cmpint(packet.data->len, ==, sizeof(raw_packet) - 1);
+
+	g_assert(0 == memcmp(packet.data->str, raw_packet, packet.data->len));
+
+	network_mysqld_auth_challenge_free(shake);
+	g_string_free(packet.data, TRUE);
+}
+
+/**
+ * 5.6.x where x = 0..2
+ * 5.5.x where x = 7..10
+ *
+ * don't send a trailing \0 after the auth method name
+ */
+static void
+test_mysqld_handshake_plugin_auth_bug_59453(void) {
+	const char raw_packet[] = 
+		"\x50\x00\x00\x00"
+		"\x0a"
+		"\x35\x2e\x36\x2e\x31\x2d\x6d\x35\x2d\x6c\x6f\x67\x00"
+		"\x4d\x00\x00\x00"
+		"\x3d\x25\x3d\x43\x76\x4d\x5e\x6d\x00"
+		"\xff\xff" /* capabilities - part 1 */
+		"\x08"     /* charset */
+		"\x02\x00" /* status */
+		"\x0f\xc0" /* capabilities - part 2 */
+		"\x15" /* auth-plugin-part-len */
+		"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" /* fillers */
+		"\x2d\x37\x69\x2d\x41\x3f\x58\x7c\x2c\x5f\x7d\x75\x00" /* auth-plugin-part-2 */
+		"\x6d\x79\x73\x71\x6c\x5f\x6e\x61\x74\x69\x76\x65\x5f\x70\x61\x73\x73\x77\x6f\x72\x64" /* mysql_native_password */
+		;
+
+	network_mysqld_auth_challenge *shake;
+	network_packet packet;
+
+	shake = network_mysqld_auth_challenge_new();
+	
+	packet.data = g_string_new(NULL);
+	packet.offset = 0;
+	g_string_append_len(packet.data, C(raw_packet));
+
+	g_assert_cmpint(packet.data->len, ==, 83);
+
+	g_assert_cmpint(0, ==, network_mysqld_proto_skip_network_header(&packet));
+	g_assert_cmpint(0, ==, network_mysqld_proto_get_auth_challenge(&packet, shake));
+
+	g_assert_cmpint(shake->server_version, ==, 50601);
+	g_assert_cmpint(shake->thread_id, ==, 77);
+	g_assert_cmpint(shake->server_status, ==,
+			SERVER_STATUS_AUTOCOMMIT);
+	g_assert_cmpint(shake->charset, ==, 8);
+	g_assert_cmphex(shake->capabilities, &,
+			(CLIENT_CONNECT_WITH_DB |
+			CLIENT_LONG_FLAG |
+
+			CLIENT_COMPRESS |
+
+			CLIENT_PROTOCOL_41 |
+
+			CLIENT_TRANSACTIONS |
+			CLIENT_SECURE_CONNECTION |
+			CLIENT_PLUGIN_AUTH));
+
+	g_assert_cmpint(shake->auth_plugin_data->len, ==, 21);
+	g_assert_cmpint(0, == ,memcmp(shake->auth_plugin_data->str, "=%=CvM^m-7i-A?X|,_}u\0", shake->auth_plugin_data->len));
+	g_assert_cmpstr(shake->auth_plugin_name->str, ==, "mysql_native_password");
+
+	/* ... and back */
+	g_string_truncate(packet.data, 0);
+	g_string_append_len(packet.data, C("P\0\0\0")); /* prepend length and sequence-id */
+	network_mysqld_proto_append_auth_challenge(packet.data, shake);
+
+	g_assert_cmpint(packet.data->len, ==, sizeof(raw_packet) - 1);
+
+	g_assert(0 == memcmp(packet.data->str, raw_packet, packet.data->len));
+
+	network_mysqld_auth_challenge_free(shake);
+	g_string_free(packet.data, TRUE);
+}
+
+
+static void test_mysqld_auth_empty_pw(void) {
 	const char raw_packet[] = 
 		"&\0\0\1\205\246\3\0\0\0\0\1\10\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0root\0\0"
 		;
 	GString *packet;
 	network_mysqld_auth_response *auth;
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(
+			CLIENT_PROTOCOL_41 |
+			CLIENT_SECURE_CONNECTION |
+			CLIENT_CONNECT_WITH_DB);
 	g_string_assign(auth->username, "root");
-	auth->capabilities    = 
+	auth->client_capabilities    = 
 		(CLIENT_LONG_PASSWORD |
 	       	CLIENT_LONG_FLAG |
 		CLIENT_LOCAL_FILES | 
@@ -323,7 +472,7 @@ void test_mysqld_auth_empty_pw(void) {
 	g_string_free(packet, TRUE);
 }
 
-void test_mysqld_auth_with_pw(void) {
+static void test_mysqld_auth_with_pw(void) {
 	const char raw_packet[] = 
 		":\0\0\1"
 		"\205\246\3\0"
@@ -340,9 +489,9 @@ void test_mysqld_auth_with_pw(void) {
 	GString *packet, *challenge, *hashed_password;
 	network_mysqld_auth_response *auth;
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(CLIENT_PROTOCOL_41 | CLIENT_SECURE_CONNECTION);
 	g_string_assign(auth->username, "root");
-	auth->capabilities    = 
+	auth->client_capabilities    = 
 		CLIENT_LONG_PASSWORD |
 	       	CLIENT_LONG_FLAG |
 		CLIENT_LOCAL_FILES | 
@@ -360,7 +509,7 @@ void test_mysqld_auth_with_pw(void) {
 	g_string_append_len(challenge, raw_challenge, sizeof(raw_challenge) - 1);
 
 	network_mysqld_proto_password_hash(hashed_password, C("123"));
-	network_mysqld_proto_password_scramble(auth->response, S(challenge), S(hashed_password));
+	network_mysqld_proto_password_scramble(auth->auth_plugin_data, S(challenge), S(hashed_password));
 
 	g_string_free(hashed_password, TRUE);
 	
@@ -393,10 +542,10 @@ void test_mysqld_auth_with_pw(void) {
  *   network_mysqld_auth_response_new() and network_mysqld_auth_response_free() 
  *   don't cause a crash 
  */
-void t_auth_response_new() {
+static void t_auth_response_new() {
 	network_mysqld_auth_response *shake;
 
-	shake = network_mysqld_auth_response_new();
+	shake = network_mysqld_auth_response_new(0);
 	g_assert(shake);
 
 	network_mysqld_auth_response_free(shake);
@@ -408,7 +557,7 @@ void t_auth_response_new() {
  *   network_mysqld_proto_append_auth_response() can encode the result
  *     of the network_mysqld_proto_get_auth_response() 
  */
-void t_mysqld_get_auth_response(void) {
+static void t_mysqld_get_auth_response(void) {
 	const char raw_packet[] = 
 		"\205\246\3\0"
 		"\0\0\0\1"
@@ -422,7 +571,7 @@ void t_mysqld_get_auth_response(void) {
 	network_packet packet;
 	int err = 0;
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(CLIENT_PROTOCOL_41 | CLIENT_SECURE_CONNECTION);
 	packet.data = g_string_new_len(C(raw_packet));
 	packet.offset = 0;
 	
@@ -434,7 +583,7 @@ void t_mysqld_get_auth_response(void) {
 	g_assert_cmpint(auth->username->len, ==, 4);
 	g_assert_cmpstr(auth->username->str, ==, "root");
 
-	g_assert_cmpuint(auth->capabilities, ==,
+	g_assert_cmpuint(auth->client_capabilities, ==,
 		CLIENT_LONG_PASSWORD |
 	       	CLIENT_LONG_FLAG |
 		CLIENT_LOCAL_FILES | 
@@ -462,7 +611,7 @@ void t_mysqld_get_auth_response(void) {
 	g_string_truncate(packet.data, 0);
 	packet.offset = 0;
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(0);
 	err = err || network_mysqld_proto_append_auth_response(packet.data, auth);
 	g_assert_cmpint(err, ==, 0);
 	network_mysqld_auth_response_free(auth);
@@ -476,7 +625,7 @@ void t_mysqld_get_auth_response(void) {
  *   network_mysqld_proto_append_auth_response() can encode the result
  *     of the network_mysqld_proto_get_auth_response() 
  */
-void t_mysqld_get_auth_response_pre_41(void) {
+static void t_mysqld_get_auth_response_pre_41(void) {
 	const char raw_packet[] = 
 		"\205$"
 		"\0\0\0"
@@ -487,7 +636,7 @@ void t_mysqld_get_auth_response_pre_41(void) {
 	network_packet packet;
 	int err = 0;
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(0);
 	packet.data = g_string_new_len(C(raw_packet));
 	packet.offset = 0;
 
@@ -499,7 +648,7 @@ void t_mysqld_get_auth_response_pre_41(void) {
 	g_assert_cmpint(auth->username->len, ==, 4);
 	g_assert_cmpstr(auth->username->str, ==, "root");
 
-	g_assert_cmpuint(auth->capabilities, ==,
+	g_assert_cmpuint(auth->client_capabilities, ==,
 		CLIENT_LONG_PASSWORD |
 	       	CLIENT_LONG_FLAG |
 		CLIENT_LOCAL_FILES | 
@@ -523,7 +672,7 @@ void t_mysqld_get_auth_response_pre_41(void) {
 	g_string_truncate(packet.data, 0);
 	packet.offset = 0;
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(0);
 	err = err || network_mysqld_proto_append_auth_response(packet.data, auth);
 	g_assert_cmpint(err, ==, 0);
 	network_mysqld_auth_response_free(auth);
@@ -533,9 +682,9 @@ void t_mysqld_get_auth_response_pre_41(void) {
 
 /**
  * @test
- *   network_mysqld_proto_get_auth_response() can decode a broken pre-4.0 packet
+ *   network_mysqld_proto_get_auth_response() can decode a pre-4.0 packet
  */
-void t_mysqld_get_auth_response_no_term(void) {
+static void t_mysqld_get_auth_response_no_term(void) {
 	const char raw_packet[] = 
 		"\205$"
 		"\0\0\0"
@@ -547,19 +696,180 @@ void t_mysqld_get_auth_response_no_term(void) {
 	network_packet packet;
 	int err = 0;
 
-	auth = network_mysqld_auth_response_new();
+	auth = network_mysqld_auth_response_new(0);
 	packet.data = g_string_new_len(C(raw_packet));
 	packet.offset = 0;
 
 	err = err || network_mysqld_proto_get_auth_response(&packet, auth);
+	g_assert_cmpint(err, ==, 0);
 
-	g_assert_cmpint(err, !=, 0);
+	g_assert_cmpstr(auth->username->str, ==, "root");
+	g_assert_cmpstr(auth->auth_plugin_data->str, ==, "foo");
 
 	network_mysqld_auth_response_free(auth);
 
 	g_string_free(packet.data, TRUE);
 }
 
+/**
+ * @test
+ *   test if CLIENT_PLUGIN_AUTH is handled correctly
+ */
+static void t_mysqld_get_auth_response_plugin_auth(void) {
+	const char raw_packet[] = 
+		"\x50\x00\x00\x01"
+		"\x85\xa6\x0f\x00" /* capabilities */
+		"\x00\x00\x00\x01" /* max packet len */
+		"\x08"             /* charset */
+		"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" /* reserved */
+		"\x72\x6f\x6f\x74\x00" /* username = root */
+		"\x14\xb6\x26\xcc\x53\x02\x62\x4d\xbf\x2c\xe8\xb4\x1e\x1d\xbf\x7d\xd6\x65\x3b\x8d\x60" /* len = 0x14 + ... */
+		"\x6d\x79\x73\x71\x6c\x5f\x6e\x61\x74\x69\x76\x65\x5f\x70\x61\x73\x73\x77\x6f\x72\x64\x00" /* mysql_native_password */
+		;
+
+	network_mysqld_auth_response *auth;
+	network_packet packet;
+	int err = 0;
+
+	auth = network_mysqld_auth_response_new(
+			CLIENT_PROTOCOL_41 |
+			CLIENT_SECURE_CONNECTION |
+			CLIENT_PLUGIN_AUTH );
+	packet.data = g_string_new_len(C(raw_packet));
+	packet.offset = 0;
+	
+	err = err || network_mysqld_proto_skip_network_header(&packet);
+	err = err || network_mysqld_proto_get_auth_response(&packet, auth);
+
+	g_assert_cmpint(err, ==, 0);
+
+	g_assert(auth->username);
+	g_assert_cmpint(auth->username->len, ==, 4);
+	g_assert_cmpstr(auth->username->str, ==, "root");
+
+	g_assert_cmphex(auth->client_capabilities, ==,
+		CLIENT_LONG_PASSWORD |
+	       	CLIENT_LONG_FLAG |
+		CLIENT_LOCAL_FILES | 
+		CLIENT_PROTOCOL_41 |
+		CLIENT_INTERACTIVE |
+		CLIENT_TRANSACTIONS |
+		CLIENT_SECURE_CONNECTION |
+		CLIENT_MULTI_STATEMENTS |
+		CLIENT_MULTI_RESULTS|
+		CLIENT_PS_MULTI_RESULTS|
+		CLIENT_PLUGIN_AUTH); 
+	g_assert_cmpuint(auth->max_packet_size, ==, 1 << 24);
+	g_assert_cmpuint(auth->charset        , ==, 8);
+	g_assert_cmpuint(auth->auth_plugin_name->len, >, 0);
+	g_assert_cmpstr(auth->auth_plugin_name->str, ==, "mysql_native_password");
+
+	g_string_truncate(packet.data, 0);
+	packet.offset = 0;
+
+	g_string_append_len(packet.data, C("\x50\x00\x00\x01"));
+	err = err || network_mysqld_proto_append_auth_response(packet.data, auth);
+	g_assert_cmpint(err, ==, 0);
+
+	g_assert_cmpint(packet.data->len, ==, sizeof(raw_packet) - 1);
+	g_assert_cmpint(TRUE, ==, g_memeq(S(packet.data), raw_packet, packet.data->len));
+
+	network_mysqld_auth_response_free(auth);
+
+	g_string_free(packet.data, TRUE);
+}
+
+#if 0
+/**
+ * get the name of a capability-flag
+ *
+ * 'flag' is to be a single bit only
+ */
+static const char* capability_flag_name(guint32 flag) {
+#define F(x) case x: return G_STRINGIFY(x);
+	switch (flag) {
+	case CLIENT_LONG_PASSWORD: return "_LONG_PASSWORD";
+	case CLIENT_FOUND_ROWS: return "_FOUND_ROWS";
+	case CLIENT_LONG_FLAG: return "_LONG_FLAG";
+	case CLIENT_CONNECT_WITH_DB: return "_CONNECT_WITH_DB";
+	case CLIENT_NO_SCHEMA: return "_NO_SCHEMA";
+	case CLIENT_COMPRESS: return "_COMPRESS";
+	case CLIENT_ODBC: return "_ODBC";
+	case CLIENT_LOCAL_FILES: return "_LOCAL_FILES";
+	case CLIENT_IGNORE_SPACE: return "_IGNORE_SPACE";
+	case CLIENT_PROTOCOL_41: return "_PROTOCOL_41";
+	case CLIENT_INTERACTIVE: return "_INTERACTIVE";
+	case CLIENT_SSL: return "_SSL";
+	case CLIENT_IGNORE_SIGPIPE: return "_IGNORE_SIGPIPE";
+	case CLIENT_TRANSACTIONS: return "_TRANSACTIONS";
+	case CLIENT_RESERVED: return "_RESERVED";
+	case CLIENT_SECURE_CONNECTION: return "_SECURE_CONNECTION";
+	case CLIENT_MULTI_STATEMENTS: return "_MULTI_STATEMENTS";
+	case CLIENT_MULTI_RESULTS: return "_MULTI_RESULTS";
+	case CLIENT_PS_MULTI_RESULTS: return "_PS_MULTI_RESULTS";
+	case CLIENT_PLUGIN_AUTH: return "_PLUGIN_AUTH";
+	case CLIENT_SSL_VERIFY_SERVER_CERT: return "_SSL_VERIFY_SERVER_CERT";
+	case CLIENT_REMEMBER_OPTIONS: return "_REMEMBER_OPTIONS";
+	}
+	return "unknown flag";
+}
+
+/**
+ * dump all capability flags that are set to "debug" 
+ */
+static void capability_flags_name(guint32 flags) {
+	int i;
+
+	for (i = 0; i < 32; i++) {
+		guint32 flag = flags & (1 << i);
+
+		if (flag) g_debug("  %s", capability_flag_name(flag));
+	}
+}
+#endif
+
+/**
+ * test if the response for the smallest set of 4.1 capabilities parses
+ * correctly: CLIENT_SECURE_CONNECTION | CLIENT_PROTOCOL_41
+ *
+ * response is taken from 'mysqltest'
+ */
+static void t_mysqld_get_auth_response_minimum_capabilities(void) {
+	guint32 capabilities;
+	const char raw_packet[] = 
+		"\x26\x00\x00\x01"
+		"\x8d\xa2\x0f\x80" /* _LONG_PASSWORD, _LONG_FLAG, _CONNECT_WITH_DB,
+				      _LOCAL_FILES,
+				      _PROTOCOL_41,
+				      _TRANSACTIONS, _SECURE_CONNECTIONS,
+				      _MULTI_STATEMENTS, _MULTI_RESULTS, _PS_MULTI_STATEMENTS, _PLUGIN_AUTH, 
+				      ...
+				      */
+
+		"\x00\x00\x00\x40"
+		"\x08"
+		"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
+		"\x72\x6f\x6f\x74\x00"
+		"\x00";
+
+	network_mysqld_auth_response *auth;
+	network_packet packet;
+	int err = 0;
+
+	auth = network_mysqld_auth_response_new(CLIENT_SECURE_CONNECTION | CLIENT_PROTOCOL_41);
+	packet.data = g_string_new_len(C(raw_packet));
+	packet.offset = 0;
+	
+	err = err || network_mysqld_proto_skip_network_header(&packet);
+	err = err || network_mysqld_proto_peek_int32(&packet, &capabilities);
+	err = err || network_mysqld_proto_get_auth_response(&packet, auth);
+
+	g_assert_cmpint(err, ==, 0);
+
+	network_mysqld_auth_response_free(auth);
+
+	g_string_free(packet.data, TRUE);
+}
 
 typedef struct {
 	const char *s;
@@ -569,7 +879,7 @@ typedef struct {
 /**
  * test that we can decode a 4.1 resultset header 
  */
-void t_resultset_fields_works(void) {
+static void t_resultset_fields_works(void) {
 	strings packets[] = {
 		{ C("\1\0\0\1\2") }, /* 2 fields */
 		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
@@ -627,7 +937,7 @@ void t_resultset_fields_works(void) {
 	network_queue_free(q);
 }
 
-void t_resultset_fields_parse_err(void) {
+static void t_resultset_fields_parse_err(void) {
 	strings packets[] = {
 		{ C("\1\0\0\1\377") }, /* err-packet */
 		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
@@ -653,7 +963,7 @@ void t_resultset_fields_parse_err(void) {
 	network_queue_free(q);
 }
 
-void t_resultset_fields_parse_null(void) {
+static void t_resultset_fields_parse_null(void) {
 	strings packets[] = {
 		{ C("\1\0\0\1\373") }, /* NULL */
 		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
@@ -679,7 +989,7 @@ void t_resultset_fields_parse_null(void) {
 	network_queue_free(q);
 }
 
-void t_resultset_fields_parse_low(void) {
+static void t_resultset_fields_parse_low(void) {
 	strings packets[] = {
 		{ C("\1\0\0\1\2") }, /* 2 fields */
 		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
@@ -704,7 +1014,7 @@ void t_resultset_fields_parse_low(void) {
 	network_queue_free(q);
 }
 
-void t_resultset_fields_parse_high(void) {
+static void t_resultset_fields_parse_high(void) {
 	strings packets[] = {
 		{ C("\1\0\0\1\2") }, /* 2 fields */
 		{ C("6\0\0\2\3def\0\6STATUS\0\rVariable_name\rVariable_name\f\10\0P\0\0\0\375\1\0\0\0\0") },
@@ -1366,6 +1676,8 @@ int main(int argc, char **argv) {
 	g_test_add_func("/core/err-packet-append", t_err_packet_append);
 
 	g_test_add_func("/core/mysqld-proto-handshake", test_mysqld_handshake);
+	g_test_add_func("/core/mysqld-proto-handshake-plugin-auth", test_mysqld_handshake_plugin_auth);
+	g_test_add_func("/core/mysqld-proto-handshake-plugin-auth-bug-59453", test_mysqld_handshake_plugin_auth_bug_59453);
 
 	g_test_add_func("/core/mysqld-proto-pw-empty", test_mysqld_auth_empty_pw);
 	g_test_add_func("/core/mysqld-proto-pw", test_mysqld_auth_with_pw);
@@ -1374,6 +1686,8 @@ int main(int argc, char **argv) {
 	g_test_add_func("/core/mysqld-proto-get-auth-response", t_mysqld_get_auth_response);
 	g_test_add_func("/core/mysqld-proto-get-auth-response-pre-4.1", t_mysqld_get_auth_response_pre_41);
 	g_test_add_func("/core/mysqld-proto-get-auth-response-no-term", t_mysqld_get_auth_response_no_term);
+	g_test_add_func("/core/mysqld-proto-get-auth-response-plugin-auth", t_mysqld_get_auth_response_plugin_auth);
+	g_test_add_func("/core/mysqld-proto-get-auth-response-minimum_capabilities", t_mysqld_get_auth_response_minimum_capabilities);
 	
 	g_test_add_func("/core/resultset-fields", t_resultset_fields_works);
 	g_test_add_func("/core/resultset-fields-broken-proto-err", t_resultset_fields_parse_err);
